//
//  AMDataLoader.m
//  UOCAlgorithms
//
//  Created by Albert Mata on 26/12/11.
//  Copyright 2011 Nem-hi Consulting. All rights reserved.
//

#import "AMDataLoader.h"
#import "AMArea.h"
#import "AMAuthor.h"
#import "AMCategory.h"
#import "AMContent.h"

@interface AMDataLoader ()
- (void)loadEntityArea;
- (void)loadEntityAuthor;
- (void)loadEntityCategory;
- (void)loadEntityContent;
- (AMArea *)areaWithName:(NSString *)name;
- (AMAuthor *)authorWithName:(NSString *)name;
- (AMCategory *)categoryWithName:(NSString *)name;
- (NSString *)markdownTextFromFile:(NSString *)textFile;
@end

@implementation AMDataLoader

/*
 * Default initilizer.
 */
- (id)init
{
    self = [super init];
    if (self) {
        // Initialization code here.
    }
    
    return self;
}

/*
 * Loads UOCAlgorithms.sqlite database.
 */
- (void)loadDatabaseUsingManagedObjectContext:(NSManagedObjectContext *)moc {
    
    managedObjectContext = moc;
    
    [self loadEntityArea];
    [self loadEntityAuthor];
    [self loadEntityCategory];
    [self loadEntityContent];
    
}

#pragma mark - Private methods

/*
 * Loads table ZAMAREA in UOCAlgorithms.sqlite database.
 */
- (void)loadEntityArea {

    // Checking areas.txt file exists and is readable.
    NSString *path = [[NSBundle mainBundle] pathForResource:@"areas" ofType:@"txt"];
    if (![[NSFileManager defaultManager] isReadableFileAtPath:path]) {
        NSLog(@"AM: Error reading file areas.txt. Aborting application.");
        exit(1);
    }
    
    // Loading and parsing areas.txt file to get array of areas.
    NSData *data = [NSData dataWithContentsOfFile:path];
    NSString *string = [[NSString alloc] initWithBytes:[data bytes]
                                                length:[data length] 
                                              encoding:NSUTF8StringEncoding];
    NSArray *items = [string componentsSeparatedByString:@"\n"];
    [string release];
    
    // Saving each area into table AMArea in database. 
    for (int i = 0; i < [items count]; i++) {
        AMArea *area = (AMArea *)
        [NSEntityDescription insertNewObjectForEntityForName:@"AMArea" 
                                      inManagedObjectContext:managedObjectContext];
        [area setName:[items objectAtIndex:i]];  
        if (![managedObjectContext save:nil]) {
            NSLog(@"AM: Error inserting area into database. Aborting application.");
            exit(1);
        }        
    }
        
}

/*
 * Loads table ZAMAUTHOR in UOCAlgorithms.sqlite database.
 */
- (void)loadEntityAuthor {    

    // Checking authors.txt file exists and is readable.
    NSString *path = [[NSBundle mainBundle] pathForResource:@"authors" ofType:@"txt"];
    if (![[NSFileManager defaultManager] isReadableFileAtPath:path]) {
        NSLog(@"AM: Error reading file authors.txt. Aborting application.");
        exit(1);
    }
    
    // Loading and parsing authors.txt file to get array of authors.
    NSData *data = [NSData dataWithContentsOfFile:path];
    NSString *string = [[NSString alloc] initWithBytes:[data bytes]
                                                length:[data length] 
                                              encoding:NSUTF8StringEncoding];
    NSArray *preitems = [string componentsSeparatedByString:@"\n"];
    [string release];
    NSMutableArray *items = [[NSMutableArray alloc] initWithCapacity:[preitems count]];
    for (int i = 0; i < [preitems count]; i++) {
        [items addObject:[[preitems objectAtIndex:i] componentsSeparatedByString:@";"]];
    }
    
    // Saving each category into table AMCategory in database. 
    for (int i = 0; i < [items count]; i++) {
        AMAuthor *author = (AMAuthor *)
        [NSEntityDescription insertNewObjectForEntityForName:@"AMAuthor" 
                                      inManagedObjectContext:managedObjectContext];
        [author setName:[[items objectAtIndex:i] objectAtIndex:0]];  
        [author setTwitter:[[items objectAtIndex:i] objectAtIndex:1]]; 
        [author setUrl:[[items objectAtIndex:i] objectAtIndex:2]];   
        if (![managedObjectContext save:nil]) {
            NSLog(@"AM: Error inserting author into database. Aborting application.");
            exit(1);
        }        
    }
    [items release];

}

/*
 * Loads table ZAMCATEGORY in UOCAlgorithms.sqlite database.
 */
- (void)loadEntityCategory {

    // Checking categories.txt file exists and is readable.
    NSString *path = [[NSBundle mainBundle] pathForResource:@"categories" ofType:@"txt"];
    if (![[NSFileManager defaultManager] isReadableFileAtPath:path]) {
        NSLog(@"AM: Error reading file categories.txt. Aborting application.");
        exit(1);
    }
    
    // Loading and parsing categories.txt file to get array of categories.
    NSData *data = [NSData dataWithContentsOfFile:path];
    NSString *string = [[NSString alloc] initWithBytes:[data bytes]
                                                length:[data length] 
                                              encoding:NSUTF8StringEncoding];
    NSArray *items = [string componentsSeparatedByString:@"\n"];
    [string release];
    
    // Saving each category into table AMCategory in database. 
    for (int i = 0; i < [items count]; i++) {
        AMCategory *category = (AMCategory *)
        [NSEntityDescription insertNewObjectForEntityForName:@"AMCategory" 
                                      inManagedObjectContext:managedObjectContext];
        [category setName:[items objectAtIndex:i]];  
        if (![managedObjectContext save:nil]) {
            NSLog(@"AM: Error inserting category into database. Aborting application.");
            exit(1);
        }        
    }    
    
}

/*
 * Loads table ZAMCONTENT in UOCAlgorithms.sqlite database.
 */
- (void)loadEntityContent {
    
    // Checking contents.txt file exists and is readable.
    NSString *path = [[NSBundle mainBundle] pathForResource:@"contents" ofType:@"txt"];
    if (![[NSFileManager defaultManager] isReadableFileAtPath:path]) {
        NSLog(@"AM: Error reading file contents.txt. Aborting application.");
        exit(1);
    }
    
    // Loading and parsing contents.txt file to get array of contents.
    NSData *data = [NSData dataWithContentsOfFile:path];
    NSString *string = [[NSString alloc] initWithBytes:[data bytes]
                                                length:[data length] 
                                              encoding:NSUTF8StringEncoding];
    NSArray *preitems = [string componentsSeparatedByString:@"\n"];
    [string release];
    NSMutableArray *items = [[NSMutableArray alloc] initWithCapacity:[preitems count]];
    for (int i = 0; i < [preitems count]; i++) {
        [items addObject:[[preitems objectAtIndex:i] componentsSeparatedByString:@";"]];
    }
    
    // Saving each content into table AMContent in database. 
    for (int i = 0; i < [items count]; i++) {
        AMContent *content = (AMContent *)
        [NSEntityDescription insertNewObjectForEntityForName:@"AMContent" 
                                      inManagedObjectContext:managedObjectContext];
        id item = [items objectAtIndex:i];
        [content setContent:[self markdownTextFromFile:[item objectAtIndex:0]]];
        [content setLanguage:[item objectAtIndex:1]];
        NSDateFormatter *df = [[NSDateFormatter alloc] init];
        [df setDateFormat:@"dd/MM/yyyy"];
        [content setPublication_date:[df dateFromString:[item objectAtIndex:2]]];
        [df release];
        [content setShort_description:[item objectAtIndex:3]];
        NSNumberFormatter *nf = [[NSNumberFormatter alloc] init];
        [content setTimes_shown:[nf numberFromString:[item objectAtIndex:4]]];
        [nf release]; 
        [content setTitle:[item objectAtIndex:5]];
        [content setArea:[self areaWithName:[item objectAtIndex:6]]];
        [content setCategory:[self categoryWithName:[item objectAtIndex:7]]];
        NSMutableSet *authors = [[NSMutableSet alloc] init];
        for (int j = 8; j < [item count]; j++) {
            [authors addObject:[self authorWithName:[item objectAtIndex:j]]];
        }
        [content setAuthors:authors]; 
        [authors release];
        if (![managedObjectContext save:nil]) {
            NSLog(@"AM: Error inserting content into database. Aborting application.");
            exit(1);
        }        
    }
    [items release];
    
}

/*
 * Returns an AMArea object based on its name.
 */
- (AMArea *)areaWithName:(NSString *)name {
    
    // Creating the request for loading an AMArea object.
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"AMArea" 
                                              inManagedObjectContext:managedObjectContext];
    [request setEntity:entity];
    
    // Creating a predicate to get just one AMArea object.
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name == %@", name];
    [request setPredicate:predicate];
    
    // Executing the request.
    NSArray *fetchResults = [managedObjectContext executeFetchRequest:request error:nil];
    [request release];
    
    // Getting the AMArea object.
    if (fetchResults == nil || [fetchResults count] == 0) {
        NSLog(@"AM: Error getting area from database. Aborting application.");
        exit(1);
    }
    return [fetchResults objectAtIndex:0];
    
}

/*
 * Returns an AMAuthor object based on its name.
 */
- (AMAuthor *)authorWithName:(NSString *)name {

    // Creating the request for loading an AMAuthor object.
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"AMAuthor" 
                                              inManagedObjectContext:managedObjectContext];
    [request setEntity:entity];
    
    // Creating a predicate to get just one AMAuthor object.
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name == %@", name];
    [request setPredicate:predicate];
    
    // Executing the request.
    NSArray *fetchResults = [managedObjectContext executeFetchRequest:request error:nil];
    [request release];
    
    // Getting the AMAuthor object.
    if (fetchResults == nil || [fetchResults count] == 0) {
        NSLog(@"AM: Error getting author from database. Aborting application.");
        exit(1);
    }
    return [fetchResults objectAtIndex:0];
    
}

/*
 * Returns an AMCategory object based on its name.
 */
- (AMCategory *)categoryWithName:(NSString *)name {
    
    // Creating the request for loading an AMCategory object.
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"AMCategory" 
                                              inManagedObjectContext:managedObjectContext];
    [request setEntity:entity];
    
    // Creating a predicate to get just one AMCategory object.
    NSPredicate *predicate = [NSPredicate predicateWithFormat:@"name == %@", name];
    [request setPredicate:predicate];
    
    // Executing the request.
    NSArray *fetchResults = [managedObjectContext executeFetchRequest:request error:nil];
    [request release];
    
    // Getting the AMCategory object.
    if (fetchResults == nil || [fetchResults count] == 0) {
        NSLog(@"AM: Error getting category from database. Aborting application.");
        exit(1);
    }
    return [fetchResults objectAtIndex:0];
    
}

/*
 * Returns a NSString with the content of a file using Markdown format (.text) in main bundle.
 */
- (NSString *)markdownTextFromFile:(NSString *)textFile {
    
    NSString *path = [[NSBundle mainBundle] pathForResource:textFile ofType:@"text"];   
    return [NSString stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:NULL];
    
}

@end
