//
//  AMMarkdownParser.m
//  UOCAlgorithms
//
//  Created by Albert Mata on 27/12/11.
//  Copyright 2011 Nem-hi Consulting. All rights reserved.
//

#import "AMMarkdownParser.h"
#import "array.h"
#import "buffer.h"
#import "markdown.h"
#import "renderers.h"

@interface AMMarkdownParser ()
- (NSString *)htmlCodeFromMarkdownText:(NSString *)markdownText;
@end

@implementation AMMarkdownParser

/*
 * Default initializer.
 */
- (id)init
{
    self = [super init];
    if (self) {
        // Initialization code here.
    }
    
    return self;
}

/*
 * Returns a NSString with full HTML code (including HEAD, CSS, BODY...) from a basic HTML code.
 */
- (NSString *)fullHtmlCodeFromMarkdownText:(NSString *)markdownText 
                              usingCssFile:(NSString *)cssFileName {
    
    return [@"<html>\n<head>\n<link href='" 
            stringByAppendingString:[cssFileName
            stringByAppendingString:[@"' rel='stylesheet' type='text/css' />\n</head>\n<body>\n" 
            stringByAppendingString:[[self htmlCodeFromMarkdownText:markdownText]
            stringByAppendingString:@"</body>\n</html>\n"]]]];
    
}

#pragma mark - Private methods

/*
 * Returns a NSString with HTML code generated from a text using Markdown format.
 */
- (NSString *)htmlCodeFromMarkdownText:(NSString *)markdownText {
    
    const char * prose = [markdownText UTF8String];  
    struct buf *ib, *ob;       
    
    int length = [markdownText lengthOfBytesUsingEncoding:NSUTF8StringEncoding] + 1;   
    
    ib = bufnew(length);
    bufgrow(ib, length);
    memcpy(ib->data, prose, length);
    ib->size = length;
    
    ob = bufnew(64);
    markdown(ob, ib, &mkd_xhtml);
    
    NSString *html = [NSString stringWithUTF8String: ob->data];
    
    bufrelease(ib);
    bufrelease(ob);
    
    return html;
    
}

@end
