//
//  DetailViewController.m
//  UOCAlgorithms
//
//  Created by Albert Mata on 25/12/11.
//  Copyright 2011 Nem-hi Consulting. All rights reserved.
//

#import "DetailViewController.h"
#import "RootViewController.h"
#import "AMMarkdownParser.h"

@interface DetailViewController ()
@property (nonatomic, retain) UIPopoverController *popoverController;
- (void)configureView;
- (void)showCredits;
- (void)showPossibleActions;
- (void)sendContentByEmail;
- (void)reportErrorInContent;
- (void)sendEmailTo:(NSString *)to withSubject:(NSString *)subject andBody:(NSString *)body;
- (void)orientationChanged:(NSNotification *)note;
@end

@implementation DetailViewController

@synthesize toolbar = _toolbar;
@synthesize activity = _activity;
@synthesize webView = _webView;
@synthesize imageView = _imageView;
@synthesize detailItem = _detailItem;
@synthesize popoverController = _myPopoverController;
@synthesize rootViewController = _rootViewController;

#pragma mark - Common view methods

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

/*
 * Customizes buttons on the top-right.
 */
- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // Registering to observe device rotation.
    UIDevice *device = [UIDevice currentDevice];
    deviceOrientation = device.orientation;
    [device beginGeneratingDeviceOrientationNotifications];
    NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
    [nc addObserver:self
           selector:@selector(orientationChanged:)
               name:UIDeviceOrientationDidChangeNotification
             object:device];
    
    // Adding buttons to the top bar.
    UIBarButtonItem *flexible = [[UIBarButtonItem alloc] 
                                 initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace 
                                 target:nil 
                                 action:nil];
    
    UIButton *infoButton = [UIButton buttonWithType:UIButtonTypeInfoDark];
    [infoButton addTarget:self 
                   action:@selector(showCredits) 
         forControlEvents:UIControlEventTouchDown];
    UIBarButtonItem *info = [[UIBarButtonItem alloc] initWithCustomView:infoButton];
    
    actionsButton = [[UIBarButtonItem alloc] 
                     initWithBarButtonSystemItem:UIBarButtonSystemItemAction 
                     target:nil 
                     action:@selector(showPossibleActions)];
    actionsButton.enabled = NO;
    
    self.toolbar.items = [NSArray arrayWithObjects:info, flexible, actionsButton, nil];
    [flexible release];
    [info release];
}

- (void)viewDidUnload
{
	[super viewDidUnload];
    
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
	self.popoverController = nil;
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

- (void)dealloc
{
    [_myPopoverController release];
    [_toolbar release];
    [_detailItem release];
    [_webView release];    
    [_imageView release];
    [_activity release];
    [actionsButton release];
    [htmlCode release];
    [super dealloc];
}

#pragma mark - Managing the detail item

/*
 * Shows content in the UIWebView and dismisses the popover controller if it's showing (portrait).
 */
- (void)setDetailItem:(NSManagedObject *)managedObject
{
	if (_detailItem != managedObject) {
		[_detailItem release];
		_detailItem = [managedObject retain];
	}
    [self configureView];
    
    if (self.popoverController != nil) {
        [self.popoverController dismissPopoverAnimated:YES];
    }		
}

#pragma mark - Split view support

/*
 * Manages how the view with the contents list is shown to the user.
 */
- (void)splitViewController:(UISplitViewController *)svc 
     willHideViewController:(UIViewController *)aViewController 
          withBarButtonItem:(UIBarButtonItem *)barButtonItem 
       forPopoverController: (UIPopoverController *)pc
{
    barButtonItem.title = @"Índex de continguts";
    NSMutableArray *items = [[self.toolbar items] mutableCopy];
    [items insertObject:barButtonItem atIndex:0];
    [self.toolbar setItems:items animated:YES];
    [items release];
    self.popoverController = pc;
}

/*
 * Manages how the view with the contents list is shown to the user.
 */
- (void)splitViewController:(UISplitViewController *)svc 
     willShowViewController:(UIViewController *)aViewController 
  invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem
{
    NSMutableArray *items = [[self.toolbar items] mutableCopy];
    [items removeObjectAtIndex:0];
    [self.toolbar setItems:items animated:YES];
    [items release];
    self.popoverController = nil;
}

#pragma mark - UIActionSheetDelegate

/*
 * Executes desired action based on UIActionSheet selection.
 */
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
    switch (buttonIndex) {
        case 0:
            [self sendContentByEmail];
            break;
        case 1:
            [self reportErrorInContent];
            break;
        default:
            break;
    }
    [actionSheet release];
}

#pragma mark - MFMailComposeViewControllerDelegate

/*
 * Controls if email has been sent or not.
 */
- (void)mailComposeController:(MFMailComposeViewController*)controller  
          didFinishWithResult:(MFMailComposeResult)result 
                        error:(NSError*)error;
{
    [self dismissModalViewControllerAnimated:YES];
}

#pragma mark - Private methods

/*
 * Shows content in the UIWebView.
 */
- (void)configureView
{
    [self.activity startAnimating];
    AMMarkdownParser *markdownParser = [[AMMarkdownParser alloc] init];
    htmlCode = [markdownParser 
                fullHtmlCodeFromMarkdownText:[[self.detailItem valueForKey:@"content"] description] 
                                usingCssFile:@"content.css"];
    [htmlCode retain];
    [markdownParser release];
    
    NSURL *mainBundleURL = [NSURL fileURLWithPath:[[NSBundle mainBundle] bundlePath]];
    [[self imageView] setHidden:YES];
    [[self webView] loadHTMLString:htmlCode baseURL:mainBundleURL];
    actionsButton.enabled = YES;
    [self.activity stopAnimating];
}

/*
 * Shows web document with application credits in the UIWebView.
 */
- (void)showCredits
{
    [self.activity startAnimating];
    NSString *path = [[NSBundle mainBundle] pathForResource:@"credits" ofType:@"html"]; 
    htmlCode = [NSString stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:nil];
    [htmlCode retain];
    NSURL *mainBundleURL = [NSURL fileURLWithPath:[[NSBundle mainBundle] bundlePath]];
    [[self imageView] setHidden:YES];
    [[self webView] loadHTMLString:htmlCode baseURL:mainBundleURL];
    actionsButton.enabled = NO;
    [self.activity stopAnimating];
    [self.rootViewController unselectAnyCell];
}

/*
 * Shows possible actions to do from currently shown content.
 */
- (void)showPossibleActions
{    
    UIActionSheet *action = [[UIActionSheet alloc] initWithTitle:nil
                                                        delegate:self
                                               cancelButtonTitle:nil
                                          destructiveButtonTitle:nil
                                               otherButtonTitles:@"Enviar per correu", 
                                                                 @"Informar d'un error", nil];
    [action showFromBarButtonItem:actionsButton animated:YES];
}

/*
 * Opens Mail.app with a preformatted mail to send current content to any destinatary.
 */
- (void)sendContentByEmail
{
    NSString *subject = [NSString stringWithFormat:@"UOC Algorithms - %@", 
                                                   [self.detailItem valueForKey:@"title"]];    
    [self sendEmailTo:nil withSubject:subject andBody:htmlCode];
}

/*
 * Opens Mail.app to send a correction about a content to author's address.
 */
- (void)reportErrorInContent
{
    NSString *to = @"mata.albert@gmail.com";
    NSString *subject = [NSString stringWithFormat:@"UOC Algorithms - %@", 
                                                   [self.detailItem valueForKey:@"title"]];  
    NSString *body = @"Expliqui aquí l'error detectat, si us plau";
    [self sendEmailTo:to withSubject:subject andBody:body];
}

/*
 * Opens Mail.app to send a preformatted mail.
 */
- (void)sendEmailTo:(NSString *)to withSubject:(NSString *)subject andBody:(NSString *)body 
{   
    MFMailComposeViewController* controller = [[MFMailComposeViewController alloc] init];
    controller.mailComposeDelegate = self;
    if (to) [controller setToRecipients:[NSArray arrayWithObject:to]];
    [controller setSubject:subject];
    [controller setMessageBody:body isHTML:YES]; 
    if (controller)
        [self presentModalViewController:controller animated:YES];   
    [controller release];
}

/*
 * Forces webView reload to avoid problems with width page (a simple reload doesn't work).
 */
- (void)orientationChanged:(NSNotification *)note
{
    UIDeviceOrientation new = [UIDevice currentDevice].orientation;

    if(new == deviceOrientation) return;
    if(new != UIDeviceOrientationPortrait &&
       new != UIDeviceOrientationPortraitUpsideDown &&
       new != UIDeviceOrientationLandscapeLeft &&
       new != UIDeviceOrientationLandscapeRight) return;
    
    deviceOrientation = new;
    if (htmlCode) {
        NSURL *mainBundleURL = [NSURL fileURLWithPath:[[NSBundle mainBundle] bundlePath]];
        [[self webView] loadHTMLString:htmlCode baseURL:mainBundleURL];
        [self.activity stopAnimating];
    }
}

@end
