//
//  RootViewController.m
//  UOCAlgorithms
//
//  Created by Albert Mata on 25/12/11.
//  Copyright 2011 Nem-hi Consulting. All rights reserved.
//

#import "RootViewController.h"
#import "DetailViewController.h"

@interface RootViewController ()
- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath;
- (NSIndexPath *)indexPathForSelectedContentTitleInSearchAfterEndSearching;
@end

@implementation RootViewController 		

@synthesize detailViewController;
@synthesize fetchedResultsController;
@synthesize managedObjectContext;
@synthesize filteredListContent;
@synthesize selectedContentTitleInSearch;
@synthesize searchIsActive;

#pragma mark - Common view methods

- (void)viewDidLoad
{
    [super viewDidLoad];
    self.title = @"UOC Algorithms";
    self.clearsSelectionOnViewWillAppear = NO;
    self.contentSizeForViewInPopover = CGSizeMake(320.0, 600.0);
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleSingleLine;
    self.tableView.backgroundColor = [UIColor colorWithRed:1.0 green:1.0 blue:1.0 
                                                     alpha:243.0 / 255.0];
    
    // Adding search display controller.
    UISearchBar *searchBar = [[UISearchBar alloc] 
                              initWithFrame:CGRectMake(0, 0, self.tableView.frame.size.width, 0)];  
	searchBar.delegate = self;  
	searchBar.showsCancelButton = YES;  
    searchBar.placeholder = @"Text a buscar";
	searchBar.scopeButtonTitles = [NSArray arrayWithObjects:@"Només títols", 
                                                            @"Contingut sencer", nil]; 
	[searchBar sizeToFit];  
	self.tableView.tableHeaderView = searchBar;  
	[searchBar release];  
    
    UISearchDisplayController *searchDisplayController = [[UISearchDisplayController alloc] 
                                                          initWithSearchBar:searchBar 
                                                          contentsController:self];  
	[self performSelector:@selector(setSearchDisplayController:) 
               withObject:searchDisplayController];
    searchDisplayController.delegate = self;  
    searchDisplayController.searchResultsDataSource = self;  
    searchDisplayController.searchResultsDelegate = self;
    [searchDisplayController release];  
    
    // Fetching data.
    NSError *error = nil;
    if (![[self fetchedResultsController] performFetch:&error]) {
        NSString *message = @"No ha estat possible recuperar el repositori de continguts "
                             "de la base de dades. Tanqui l'aplicació, si us plau.";
        UIAlertView *a = [[[UIAlertView alloc] initWithTitle:@"Error recuperant dades"
                                                     message:message
                                                    delegate:nil
                                           cancelButtonTitle:@"OK"
                                           otherButtonTitles:nil] autorelease];
        [a show];
    }
    self.filteredListContent = [NSMutableArray arrayWithCapacity:
                                [[[self fetchedResultsController] fetchedObjects] count]];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    // Relinquish ownership any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload
{
    self.searchIsActive = [self.searchDisplayController isActive];
	self.filteredListContent = nil;
	self.fetchedResultsController = nil;
}

- (void)dealloc
{
    [detailViewController release];
    [fetchedResultsController release];
    [managedObjectContext release];
    [filteredListContent release];
    [selectedContentTitleInSearch release];
    [super dealloc];
}

#pragma mark - Content filtering

/*
 * Filters content based on text introduced by the user.
 */
- (void)filterContentForSearchText:(NSString*)searchText scope:(NSString*)scope
{
	if ([scope isEqualToString:@"Només títols"]) {
		NSPredicate * predicate = [NSPredicate predicateWithFormat:@"title CONTAINS[cd] %@", 
                                                                   searchText];
		self.filteredListContent = [[[self fetchedResultsController] fetchedObjects] 
                                    filteredArrayUsingPredicate:predicate];
	} else {
		NSPredicate * predicate = [NSPredicate predicateWithFormat:@"content CONTAINS[cd] %@", 
                                                                   searchText];
		self.filteredListContent = [[[self fetchedResultsController] fetchedObjects] 
                                    filteredArrayUsingPredicate:predicate];
	}
}

#pragma mark - UISearchDisplayDelegate

/*
 * Updates results after user has introduced some new text on search field.
 */
- (BOOL)searchDisplayController:(UISearchDisplayController *)controller 
shouldReloadTableForSearchString:(NSString *)searchString
{
    [self filterContentForSearchText:searchString scope:
	 [[self.searchDisplayController.searchBar scopeButtonTitles] objectAtIndex:
	  [self.searchDisplayController.searchBar selectedScopeButtonIndex]]];
    
    return YES;
}

/*
 * Updates results after user has pressed an option button below search field.
 */
- (BOOL)searchDisplayController:(UISearchDisplayController *)controller 
shouldReloadTableForSearchScope:(NSInteger)searchOption
{
    [self filterContentForSearchText:[self.searchDisplayController.searchBar text] scope:
	 [[self.searchDisplayController.searchBar scopeButtonTitles] objectAtIndex:searchOption]];
    
    return YES;
}

/*
 * Sets shown results are under an active search.
 */
- (void)searchDisplayControllerWillBeginSearch:(UISearchDisplayController *)controller {
	self.searchIsActive = YES;
}

/*
 * Sets shown results are NOT under an active serch.
 */
- (void)searchDisplayControllerDidEndSearch:(UISearchDisplayController *)controller {
	self.searchIsActive = NO;
    NSIndexPath *rowToSelect = [self indexPathForSelectedContentTitleInSearchAfterEndSearching];
    NSLog(@"NSIndexPath, SECTION = %i, ROW = %i",[rowToSelect section], [rowToSelect row]);
    [[self tableView] selectRowAtIndexPath:rowToSelect 
                                  animated:YES 
                            scrollPosition:UITableViewScrollPositionNone];
    [[self tableView] scrollToRowAtIndexPath:rowToSelect 
                            atScrollPosition:UITableViewScrollPositionNone 
                                    animated:YES];    
}

#pragma mark - NSFetchedResultsControllerDelegate

/*
 * Fetchs database to get content grouped by areas.
 */
- (NSFetchedResultsController *)fetchedResultsController
{
    if (fetchedResultsController != nil) {
        return fetchedResultsController;
    }
    
    // Creating the fetch request for the contents.
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"AMContent" 
                                              inManagedObjectContext:managedObjectContext];
    [fetchRequest setEntity:entity];
    
    // Setting the batch size to a suitable number.
    [fetchRequest setFetchBatchSize:20];
    
    // Sorting the contents by area and title, both alphabetically.
    NSSortDescriptor *sd1 = [[NSSortDescriptor alloc] initWithKey:@"area.name" ascending:YES];
    NSSortDescriptor *sd2 = [[NSSortDescriptor alloc] initWithKey:@"title" ascending:YES];
    NSArray *sortDescriptors = [[NSArray alloc] initWithObjects:sd1, sd2, nil];
    [sd1 release];
    [sd2 release];
    [fetchRequest setSortDescriptors:sortDescriptors];
    
    // Edit the section name key path and cache name if appropriate.
    // nil for section name key path means "no sections".
    NSFetchedResultsController *aFetchedResultsController = 
        [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest 
                                            managedObjectContext:managedObjectContext 
                                              sectionNameKeyPath:@"area.name" 
                                                       cacheName:@"ContentsCache"];
    aFetchedResultsController.delegate = self;
    self.fetchedResultsController = aFetchedResultsController;
    
    [aFetchedResultsController release];
    [fetchRequest release];
    
    [sortDescriptors release];
    
    return fetchedResultsController;
}    
 
#pragma mark - UITableViewDataSource

/*
 * Returns number of sections.
 */
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    if (self.searchIsActive) {
        return 1;
    }
    return [[fetchedResultsController sections] count];
}

/*
 * Returns titles for sections.
 */
- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    if (self.searchIsActive) {
        return @"Resultats de la cerca";
    }
    return [[[fetchedResultsController sections] objectAtIndex:section] name];
}

/*
 * Returns number of rows in each section.
 */
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    if (self.searchIsActive) {
        return [self.filteredListContent count];
    }
    
    return [[[fetchedResultsController sections] objectAtIndex:section] numberOfObjects];
}

/*
 * Returns UITableViewCell object for a given section and row number.
 */
- (UITableViewCell *)tableView:(UITableView *)tableView 
         cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *CellIdentifier = @"AMTableViewCell";
    
    AMTableViewCell *cell = (AMTableViewCell *) [tableView 
                                                 dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {       
        NSArray *nib = [[NSBundle mainBundle] loadNibNamed:CellIdentifier owner:self options:nil];
        cell = (AMTableViewCell *)[nib objectAtIndex:0];
    }
    
    // Configuring the cell.
    [self configureCell:cell atIndexPath:indexPath];
    return cell;
}

/*
 * Sets if rows can be reordered or not.
 */
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    return NO;
}

/*
 * Sets if rows can be edited or not.
 */
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    return NO;
}

#pragma mark - UITableViewDelegate

/*
 * Shows detail of content when it's been selected on TableView.
 */
- (void)tableView:(UITableView *)aTableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Setting the detail item in the detail view controller.
    NSManagedObject *selectedObject = nil;
     
    if (self.searchIsActive) {
		selectedObject = [[self filteredListContent] objectAtIndex:[indexPath row]];
        selectedContentTitleInSearch = [selectedObject valueForKey:@"title"];
	} else
		selectedObject = [fetchedResultsController objectAtIndexPath:indexPath];
    detailViewController.detailItem = selectedObject;    
}

#pragma mark - Public methods

/*
 * Unselects all cells in TableView so previously selected one turns unselected.
 */
-(void)unselectAnyCell {
    [[self tableView] selectRowAtIndexPath:nil 
                                  animated:YES 
                            scrollPosition:UITableViewScrollPositionNone];
}

#pragma mark - Private methods

/*
 * Configures a cell just by setting its main text to content's title.
 */
- (void)configureCell:(AMTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath
{
    NSManagedObject *managedObject = nil;
	if (self.searchIsActive) {
		managedObject = [[self filteredListContent] objectAtIndex:[indexPath row]];
	} else {
		managedObject = [fetchedResultsController objectAtIndexPath:indexPath];
    }
    
	cell.contentTitle.text = [managedObject valueForKey:@"title"];
    cell.contentTitle.highlightedTextColor = [UIColor whiteColor];
    
    // Setting a blue gradient background for selected cell. 
    UIView *view = [[[UIView alloc] initWithFrame:[cell.backgroundView frame]] autorelease];
    CAGradientLayer *gradient = [CAGradientLayer layer];
    gradient.frame = view.bounds;
    gradient.colors = [NSArray arrayWithObjects:
                       (id)[[UIColor colorWithRed:0.6 green:0.8 blue:1.0 alpha:1.0] CGColor], 
                       (id)[[UIColor colorWithRed:0.4 green:0.6 blue:0.8 alpha:1.0] CGColor], nil];
    [view.layer insertSublayer:gradient atIndex:0];
    cell.selectedBackgroundView = view;
}

/*
 * Returns NSIndexPath corresponding to last selected content in a search.
 */
- (NSIndexPath *)indexPathForSelectedContentTitleInSearchAfterEndSearching
{
    for (id object in [fetchedResultsController fetchedObjects]) {
        if ([[((NSManagedObject *) object) valueForKey:@"title"] 
             isEqualToString:selectedContentTitleInSearch])
            return [fetchedResultsController indexPathForObject:object];
    }
    return nil;
}

@end
