//
//  UOCAlgorithmsAppDelegate.m
//  UOCAlgorithms
//
//  Created by Albert Mata on 25/12/11.
//  Copyright 2011 Nem-hi Consulting. All rights reserved.
//

#import "UOCAlgorithmsAppDelegate.h"
#import "RootViewController.h"
#import "DetailViewController.h"
#import "AMDataLoader.h"

@implementation UOCAlgorithmsAppDelegate

@synthesize window = _window;
@synthesize managedObjectContext = __managedObjectContext;
@synthesize managedObjectModel = __managedObjectModel;
@synthesize persistentStoreCoordinator = __persistentStoreCoordinator;
@synthesize splitViewController = _splitViewController;
@synthesize rootViewController = _rootViewController;
@synthesize detailViewController = _detailViewController;

#pragma mark - Main application methods

/*
 * Manages first application launching.
 */
- (BOOL)application:(UIApplication *)application 
didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{    
    // NO in release versions, YES when we want to populate database from .text files.
    // BOOL wantPopulateDatabase = NO;
    // if (wantPopulateDatabase)
    //     [self populateDatabaseFromTextFiles];
    
    [self populateDatabaseFromTextFiles];
    
    // Adding the split view controller's view to the window and display.
    self.window.rootViewController = self.splitViewController;
    [self.window makeKeyAndVisible];
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    /*
     Sent when the application is about to move from active to inactive state. This can occur for 
     certain types of temporary interruptions (such as an incoming phone call or SMS message) or 
     when the user quits the application and it begins the transition to the background state.
     Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame 
     rates. Games should use this method to pause the game.
     */
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    /*
     Use this method to release shared resources, save user data, invalidate timers, and store 
     enough application state information to restore your application to its current state in case 
     it is terminated later. If your application supports background execution, this method is 
     called instead of applicationWillTerminate: when the user quits.
     */
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    /*
     Called as part of the transition from the background to the inactive state; here you can undo 
     many of the changes made on entering the background.
     */
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    /*
     Restart any tasks that were paused (or not yet started) while the application was inactive. If 
     the application was previously in the background, optionally refresh the user interface.
     */
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    // Saves changes in the application's managed object context before the application terminates.
    [self saveContext];
}

- (void)dealloc
{
    [_window release];
    [__managedObjectContext release];
    [__managedObjectModel release];
    [__persistentStoreCoordinator release];
    [_splitViewController release];
    [_rootViewController release];
    [_detailViewController release];
    [super dealloc];
}

- (void)awakeFromNib
{
    // Pass the managed object context to the root view controller.
    self.rootViewController.managedObjectContext = self.managedObjectContext; 
}
- (void)saveContext
{
    NSError *error = nil;
    NSManagedObjectContext *managedObjectContext = self.managedObjectContext;
    if (managedObjectContext != nil)
    {
        if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error])
        {
            NSString *message = @"Classe: UOCAlgorithmsAppDelegate\n"
                                 "Mètode: saveContext";
            UIAlertView *a = [[[UIAlertView alloc] initWithTitle:@"Error inesperat"
                                                         message:message
                                                        delegate:nil
                                               cancelButtonTitle:@"OK"
                                               otherButtonTitles:nil] autorelease];
            [a show];
            abort();
        } 
    }
}

#pragma mark - Core Data stack

/*
 * Returns the managed object context for the application. If the context doesn't already exist, 
 * it is created and bound to the persistent store coordinator for the application.
 */
- (NSManagedObjectContext *)managedObjectContext
{
    if (__managedObjectContext != nil)
    {
        return __managedObjectContext;
    }
    
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil)
    {
        __managedObjectContext = [[NSManagedObjectContext alloc] init];
        [__managedObjectContext setPersistentStoreCoordinator:coordinator];
    }
    return __managedObjectContext;
}

/*
 * Returns the managed object model for the application. If the model doesn't already exist, it is 
 * created from the application's model.
 */
- (NSManagedObjectModel *)managedObjectModel
{
    if (__managedObjectModel != nil)
    {
        return __managedObjectModel;
    }
    NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"UOCAlgorithms" withExtension:@"momd"];
    __managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];    
    return __managedObjectModel;
}

/*
 * Returns the persistent store coordinator for the application. If the coordinator doesn't already 
 * exist, it is created and the application's store added to it.
 */
- (NSPersistentStoreCoordinator *)persistentStoreCoordinator
{
    if (__persistentStoreCoordinator != nil)
    {
        return __persistentStoreCoordinator;
    }
    
    NSURL *storeURL = [[self applicationDocumentsDirectory] 
                       URLByAppendingPathComponent:@"UOCAlgorithms.sqlite"];
    
    NSError *error = nil;
    __persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] 
                                    initWithManagedObjectModel:[self managedObjectModel]];
    if (![__persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType 
                                                    configuration:nil 
                                                              URL:storeURL 
                                                          options:nil 
                                                            error:&error])
    {
        NSString *message = @"Classe: UOCAlgorithmsAppDelegate\n"
                             "Mètode: persistentStoreCoordinator";
        UIAlertView *a = [[[UIAlertView alloc] initWithTitle:@"Error inesperat"
                                                     message:message
                                                    delegate:nil
                                           cancelButtonTitle:@"OK"
                                           otherButtonTitles:nil] autorelease];
        [a show];
        abort();
    }    
    
    return __persistentStoreCoordinator;
}

#pragma mark - Application's Documents directory

/*
 * Returns the URL to the application's Documents directory.
 */
- (NSURL *)applicationDocumentsDirectory
{
    return [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory 
                                                   inDomains:NSUserDomainMask] lastObject];
}

#pragma mark - SQLite database creation

/*
 * Creates a new database file with content from text files.
 */
- (void)populateDatabaseFromTextFiles
{
    NSPersistentStoreCoordinator *storeCoordinator = [self persistentStoreCoordinator];
    NSPersistentStore *store = [[storeCoordinator persistentStores] objectAtIndex:0];
    NSURL *storeURL = store.URL;
    
    // Deleting current UOCAlgorithms.sqlite database file.
    [storeCoordinator removePersistentStore:store error:nil];
    [[NSFileManager defaultManager] removeItemAtURL:storeURL error:nil];
    
    // Creating UOCAlgorithms.sqlite database file again.
    [storeCoordinator addPersistentStoreWithType:NSSQLiteStoreType 
                                   configuration:nil URL:storeURL options:nil error:nil];
    
    // Populating database.
    AMDataLoader *dataLoader = [[AMDataLoader alloc] init];
    [dataLoader loadDatabaseUsingManagedObjectContext:self.managedObjectContext];
    [dataLoader release];
    //exit(0);    
}

@end
